// Copyright IBM Corp. 2015,2016. All Rights Reserved.
// Node module: wlpn-cli-scaling-member
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

'use strict';

var ScalingMember = require('../lib/scaling-member');
var tap = require('tap');

tap.test('scaling-member', function(t) {
  var options = {
    serverName: 'testServer',
    serverPath: '/home/user/wlpn/testServer',
    adminPort: 9081,
    userDir: '/home/user/wlpn',
    hostName: 'system.raleigh.ibm.com',
    logger: {
      log: function() {},
      error: function() {},
    },
  };
  var sentMessage = null;
/*
 this._controller = manager.get('controller')();
  this._runner = manager.get('runner')();
  this._manager = manager;
  */
  var runnerCallback;

  options.manager = {
    get: function(name) {
      if (name === 'runner') {
        return function() {
          return {
            on: function(label, cb) {
              runnerCallback = cb;
            },
          };
        };
      } else if (name === 'controller') {
        return function() {
          return {
            request: function(msg, cb) { sentMessage = msg; cb(); },
          };
        };
      };
    },
    app: {
      use: function() {},
    },
  };

  t.test('test-start', function(t) {
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    t.end();
  });

  t.test('send-message', function(t) {
    options.manager.get();
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    sentMessage = null;
    member.sendMessage('message', callback);
    if (sentMessage == null) {
      t.fail(sentMessage);
    } else {
      t.pass(sentMessage);
    }
    t.end();
  });

  t.test('receive-threshold-messages', function(t) {
    options.manager.get();
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    sentMessage = null;
    var message = {
      messageBody: {
        containers: {},
        host: true,
        metrics: {
          heap: {
            thresholds: {},
          },
          cpu: {
            thresholds: {},
          },
          memory: {
            thresholds: {},
          },
        },
      },
    };
    message.messageBody.metrics.cpu.thresholds[0] = {
      upper: true,
      value: 100,
    };
    message.messageBody.metrics.memory.thresholds[0] = {
      upper: true,
      value: 100,
    };
    var serverName = 'server/system.raleigh.ibm.com,' +
        '%2Fhome%2Fuser%2Fwlpn,testServer';
    message.messageBody.containers[serverName] = {
      metrics: {
        cpu: {
          thresholds: {},
        },
        heap: {
          thresholds: {},
        },
        memory: {
          thresholds: {},
        },
      },
    };
    // Setting all the thresholds to increase test coverage
    message.messageBody.containers[serverName].metrics.cpu.thresholds[0] = {
      upper: true,
      value: 100,
    };
    message.messageBody.containers[serverName].metrics.cpu.thresholds[1] = {
      upper: false,
      value: 0,
    };
    message.messageBody.containers[serverName].metrics.heap.thresholds[0] = {
      upper: true,
      value: 100,
    };
    message.messageBody.containers[serverName].metrics.heap.thresholds[1] = {
      upper: false,
      value: 0,
    };
    message.messageBody.containers[serverName].metrics.memory.thresholds[0] = {
      upper: true,
      value: 100,
    };
    message.messageBody.containers[serverName].metrics.memory.thresholds[1] = {
      upper: false,
      value: 0,
    };
    member.receiveMessage(member, message);
    // Check that we don't have a threshold breach
    t.equal(member.checkThresholds(), false);
	// Update threshold to be at 0 to cause a breach
    message.messageBody.containers[serverName].metrics.cpu.thresholds[0] = {
      upper: true,
      value: -1,
    };
    message.messageBody.containers[serverName].metrics.cpu.thresholds[1] = {};
    member.receiveMessage(member, message);
   //  Detect threshold breach
    t.equal(member.checkThresholds(), true);

    t.end();
  });

  t.test('test-messaging', function(t) {
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    t.assert(member.initialUpMessage());
	// send message init
    var initialized = {
      cpu: true,
      memory: true,
      heap: true,
      all: true,
      sent: false,
    };
    var hostMax = {};
    var serverMax = {};
    t.assert(member._metricsInitializedMessage(hostMax, serverMax,
                                               initialized));
    var hostUsed = {};
    var serverUsed = {};
    t.assert(member._metricsMessage(hostUsed, serverUsed, hostMax, serverMax));
    t.end();
  });

  t.test('test-reconnect', function(t) {
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    member.reconnect(server, callback);
    t.assert(member);
    t.end();
  });

  t.test('test-appmetrics', function(t) {
    var member = new ScalingMember(options);
    var server = {};
    var callback = function() {};
    member.start(server, callback);
    t.assert(member);
    // Send a cpu message -
    var appMessage = {
      cmd: 'appmetrics:cpu',
      data: {
        process: 15,
        system: 15,
      },
    };
    runnerCallback(appMessage);
    t.assert(member);

    // Then memory
    appMessage.cmd = 'appmetrics:memory';
    appMessage.data = {
      // the total amount of RAM in use on the system in bytes.
      physical: 50,
      // the amount of RAM used by the Node.js application in bytes.
      physical_used: 50,
      physical_total: 100,
    };
    runnerCallback(appMessage);
    t.assert(member);

    // Then heap
    appMessage.cmd = 'appmetrics:gc';
    appMessage.data = {
      // the amount of memory used on the JavaScript heap in bytes
      used: 50,
      size: 100,
    };
    runnerCallback(appMessage);
    t.assert(member);

    t.end();
  });

  t.end();
});
